# podflow/upload/time_key.py
# coding: utf-8

import hashlib
import base64
from datetime import datetime


# 生成时间密钥模块
def time_key(seed, time_interval=300, next_time_zone=False):  # 900 秒 = 15 分钟
    # 获取当前时间戳
    current_time = datetime.now()
    # 计算时间间隔的时间单元
    time_unit = int(current_time.timestamp() // time_interval)
    if next_time_zone:
        time_unit += 1
    # 将时间单元和种子组合生成密钥
    key_input = f"{time_unit}{seed}".encode("utf-8")
    # 使用 SHA-256 哈希算法生成密钥
    hashed_key = hashlib.sha256(key_input).digest()
    # 将密钥编码为 base64 格式
    encoded_key = base64.b64encode(hashed_key).decode("utf-8")
    return encoded_key


# 时间密钥校验模块
def check_time_key(key, seed, time_interval=300):
    key_list = [
        time_key(seed, time_interval),
        time_key(seed, time_interval, True),
    ]
    return key in key_list



# podflow/basic/get_file_list.py
# coding: utf-8

import os
import re
from podflow.basic.list_merge_tidy import list_merge_tidy


# 获取文件列表和分P列表
def get_file_list(video_key, video_media="m4a", length=12):
    media = ("m4a", "mp4", "part") if video_media == "m4a" else ("mp4", "part")
    try:
        content_id = [
            file  # 获取文件名（包括扩展名）
            for file in os.listdir(
                f"channel_audiovisual/{video_key}"
            )  # 遍历指定目录下的所有文件
            if file.endswith(media)  # 筛选出以 media 结尾的文件
        ]
        content_id_items = []
        items_counts = {}
        for id_num in content_id:
            if len(id_num) > length + 4:
                content_id_items.append(id_num)
            if ".part" in id_num:
                items_counts[id_num[:length]] = 0
        content_id_items = [id_num[:length] for id_num in content_id_items]
        for id_num in content_id_items:
            if id_num not in items_counts:
                qtys = content_id_items.count(id_num)
                if video_media == "m4a":
                    pattern = re.compile(rf"{id_num}_[0-9]*\.(m4a|mp4)")
                else:
                    pattern = re.compile(rf"{id_num}_[0-9]*\.(mp4)")
                if len([item for item in content_id if pattern.search(item)]) == qtys:
                    items_counts[id_num] = qtys
                else:
                    fail = False
                    for qty in range(qtys):
                        if video_media == "m4a":
                            if (
                                f"{id_num}_p{qty + 1}.m4a" not in content_id
                                and f"{id_num}_p{qty + 1}.mp4" not in content_id
                            ):
                                fail = True
                        elif f"{id_num}_p{qty + 1}.mp4" not in content_id:
                            fail = True
                    items_counts[id_num] = 0 if fail else qtys
        content_id = list_merge_tidy(content_id, [], length)
        for id_num, num in items_counts.copy().items():
            if num in [1, 0] and id_num in content_id:
                content_id.remove(id_num)
                del items_counts[id_num]
    except Exception:
        content_id = []
        items_counts = {}
    return content_id, items_counts



# podflow/basic/get_html_dict.py
# coding: utf-8

import re
import json
from bs4 import BeautifulSoup
from podflow.basic.http_client import http_client


# 通过bs4获取html中字典模块
def get_html_dict(url, name, script_label):
    if not (response := http_client(url, name)):
        return None
    html_content = response.text
    # 使用Beautiful Soup解析HTML
    soup = BeautifulSoup(html_content, 'html.parser')
    if not (
        data_script := soup.find(
            'script', string=lambda t: t and script_label in t
        )
    ):
        return None
    try:
        # 使用正则表达式提取 JSON 数据
        pattern = re.compile(r'\{.*\}', re.DOTALL)
        if match := pattern.search(data_script.text):
            data_str = match.group()
            return json.loads(data_str)
    except json.JSONDecodeError:
        return None



# podflow/httpfs/download_bar.py
# coding: utf-8

from podflow import gVar


def download_bar(
    mod=0,
    per=0,
    retime="00:00",
    speed="   0.00 B",
    part="",
    status="准备中",
    idname="",
    nametext="",
    file="",
):
    if mod == 0:
        gVar.index_message["download"].append(
            [
                per,
                retime,
                speed,
                part,
                status,
                idname,
                nametext,
                file,
            ]
        )
    elif mod == 1 and gVar.index_message["download"]:
        gVar.index_message["download"][-1][0] = per
        gVar.index_message["download"][-1][1] = retime
        gVar.index_message["download"][-1][2] = speed
        gVar.index_message["download"][-1][3] = part
        gVar.index_message["download"][-1][4] = status
    elif mod == 2 and gVar.index_message["download"]:
        gVar.index_message["download"][-1][4] = status



# podflow/repair/reverse_log.py
# coding: utf-8

import re
from itertools import islice
from podflow.basic.time_print import time_print


def reverse_log(filename):
    try:
        with open(f"{filename}.log", "r", encoding="utf-8") as file:
            lines = list(islice(file, 10))
    except Exception:
        return
    num = 0
    end_num = len(lines) - 1

    def date_time(num):
        pattern = r"^([0-9]{4})-(0[1-9]|1[0-2])-(0[1-9]|[12][0-9]|3[01])\s(0[0-9]|1[0-9]|2[0-3]):([0-5][0-9]):([0-5][0-9])$"
        date_str = lines[num][:19]
        return re.match(pattern, date_str)

    while not date_time(num):
        num += 1
    while not date_time(end_num):
        end_num -= 1
    if end_num > num and lines[num][:19] > lines[end_num][:19]:
        with open(f"{filename}.log", "r", encoding="utf-8") as file:
            lines = file.readlines()
            # 反转行的顺序
            reversed_lines = lines[::-1]
        with open(f"{filename}.log", "w", encoding="utf-8") as file:
            file.writelines(reversed_lines)
            time_print(f"{filename}.log反转成功")



# podflow/basic/qr_code.py
# coding: utf-8

import math
import pyqrcode
from podflow.basic.time_print import time_print


# 网址二维码模块
def qr_code(
    data,
    to_html=False,
    num=None
):
    qr = pyqrcode.create(
        data,
        error='L',  # 对应于ERROR_CORRECT_L，可选值: 'L','M','Q','H'
    )
    # 获取QR码矩阵（作为二维数组）
    matrix = qr.code    # 获取图像的宽度和高度
    width, height = len(matrix), len(matrix)
    height_double = math.ceil(height / 2)
    # 转换图像为ASCII字符
    fonts = ["▀", "▄", "█", " "]
    ascii_art = ""
    for y in range(height_double):
        if (y + 1) * 2 - 1 >= height:
            for x in range(width):
                ascii_art += (
                    fonts[0] if matrix[(y + 1) * 2 - 2][x] == 1 else fonts[3]
                )
        else:
            for x in range(width):
                if (
                    matrix[(y + 1) * 2 - 2][x] == 1
                    and matrix[(y + 1) * 2 - 1][x] == 1
                ):
                    ascii_art += fonts[2]
                elif (
                    matrix[(y + 1) * 2 - 2][x] == 1
                    and matrix[(y + 1) * 2 - 1][x] == 0
                ):
                    ascii_art += fonts[0]
                elif (
                    matrix[(y + 1) * 2 - 2][x] == 0
                    and matrix[(y + 1) * 2 - 1][x] == 1
                ):
                    ascii_art += fonts[1]
                else:
                    ascii_art += fonts[3]
            ascii_art += "\n"
    Qrcode = data if to_html else ""
    time_print(
        ascii_art,
        Time=False,
        Qrcode=Qrcode,
        Number=num,
    )
    return height_double



# podflow/httpfs/to_html.py
# coding: utf-8

import re
import html


def ansi_to_html(ansi_text):
    html_output = ""
    ansi_codes = {
        "\033[30m": "ansi-30m",  # 黑色
        "\033[31m": "ansi-31m",  # 红色
        "\033[32m": "ansi-32m",  # 绿色
        "\033[33m": "ansi-33m",  # 黄色
        "\033[34m": "ansi-34m",  # 蓝色
        "\033[35m": "ansi-35m",  # 品红
        "\033[36m": "ansi-36m",  # 青色
        "\033[37m": "ansi-37m",  # 白色
        "\033[90m": "ansi-90m",  # 亮黑色
        "\033[91m": "ansi-91m",  # 亮红色 
        "\033[92m": "ansi-92m",  # 亮绿色
        "\033[93m": "ansi-93m",  # 亮黄色
        "\033[94m": "ansi-94m",  # 亮蓝色
        "\033[95m": "ansi-95m",  # 亮品红
        "\033[96m": "ansi-96m",  # 亮青色
        "\033[97m": "ansi-97m",  # 亮白色
        "\033[0m": "",  # 重置
    }
    inside_span = False

    parts = re.split(r"(\033\[\d+m)", ansi_text)

    for part in parts:
        if part in ansi_codes:
            if style := ansi_codes[part]:
                if inside_span:
                    html_output += "</span>"
                html_output += f'<span class="{style}">'
                inside_span = True
            elif inside_span:  # Reset code
                html_output += "</span>"
                inside_span = False
        else:
            escaped_part = html.escape(part)
            html_output += escaped_part

    if inside_span:
        html_output += "</span>"
    html_output = re.sub(r'^\s', '', html_output)
    html_output = re.sub(r'\s+$', '', html_output)
    return html_output


def qrcode_to_html(url):
    return f'<span class="qrcode-container" data-url="{url}"></span>'



# podflow/upload/upload_files.py
# coding: utf-8

import time
from datetime import datetime
from podflow import gVar
from podflow.httpfs.to_html import ansi_to_html
from podflow.upload.build_hash import build_hash
from podflow.basic.http_client import http_client
from podflow.httpfs.app_bottle import bottle_app_instance


# 上传文件模块
def upload_file(username, password, channelid, filename):
    filename = f"channel_audiovisual/{channelid}/{filename}"
    with open(filename, "rb") as file:
        file.seek(0)
        hashs = build_hash(file)
        file.seek(0)
        data = {
            "username": username,
            "password": password,
            "channel_id": channelid,
            "hash": hashs,
        }
        if response := http_client(
            url="http://10.0.3.231:5000/upload",
            name="1",
            data=data,
            mode="post",
            file=file,
        ):
            return response.json(), hashs
        else:
            return None, hashs
    return None, hashs


# 查找位置模块
def find_media_index(upload_original, target_media_id):
    for index, item in enumerate(upload_original):
        if item.get("media_id") == target_media_id:
            return index  # 返回找到的索引
    return -1


# 过滤和排序媒体模块
def filter_and_sort_media(media_list, day):
    current_time = int(time.time())
    one_month_ago = current_time - day * 24 * 60 * 60  # 30天前的时间戳
    filtered_sorted = sorted(
        (
            item
            for item in media_list
            if not item["upload"]
            and not item["remove"]
            and item["media_time"] < one_month_ago
        ),
        key=lambda x: x["media_time"],
    )
    result = [
        {"media_id": item["media_id"], "channel_id": item["channel_id"]}
        for item in filtered_sorted
    ]
    return result


def record_upload(username, password, channelid, filename):
    response, hashs = upload_file(username, password, channelid, filename)
    channelname = (
        gVar.channelid_youtube_ids_original | gVar.channelid_bilibili_ids_original
    ).get(channelid, "")
    now_time = datetime.now().strftime("%H:%M:%S")
    if response:
        code = response.get("code")
        data = response.get("data", {})
        message = response.get("message", "")
        if code in [0, 1]:
            index = find_media_index(gVar.upload_original, filename)
            if index != -1:
                filename = data.get("filename")
                if filename:
                    gVar.upload_original[index]["upload"] = True
                    gVar.upload_original[index]["hash"] = hashs
                    gVar.upload_original[index]["filename"] = hashs

        bottle_text = f"{now_time}|{channelname}/{filename} Upload: {message}"
    else:
        bottle_text = f"{now_time}|{channelname}/{filename} Upload Failed"
    bottle_app_instance.bottle_print.append(bottle_text)
    gVar.index_message["http"].append(ansi_to_html(bottle_text))
    bottle_app_instance.cherry_print(False)



# podflow/upload/linked_client.py
# coding: utf-8

import time
import socket
import struct
import threading
from podflow import gVar
from podflow.upload.time_key import time_key
from podflow.basic.time_print import time_print
from podflow.httpfs.progress_bar import progress_update


BROADCAST_PORT = 37001
TIMEOUT = 1  # 搜索超时时间（秒）
MAX_BROADCAST_PORT = 37010  # 尝试广播的最大端口


# 获取本机局域网 IP (此函数在有代理时可能失效，但保留作为fallback或无代理时的功能)
def get_local_ip():
    try:
        # 尝试连接一个外部地址来确定哪个本地接口被用于路由
        # 注意：在某些代理/VPN配置下，这可能返回代理或VPN分配的IP
        with socket.socket(socket.AF_INET, socket.SOCK_DGRAM) as s:
            s.connect(("8.8.8.8", 80))  # 非真实连接
            ip = s.getsockname()[0]
            time_print(f"局域网IP:{ip}")
            return ip
    except Exception as e:
        time_print(f"\033[31m获取本地 IP 失败:\033[0m {e}")
        return None


# 计算广播地址（默认 255.255.255.0）
def calculate_broadcast(ip, netmask="255.255.255.0"):
    try:
        # 确保输入是有效的IPv4地址
        socket.inet_aton(ip)
        socket.inet_aton(netmask)

        ip_packed = struct.unpack("!I", socket.inet_aton(ip))[0]
        mask_packed = struct.unpack("!I", socket.inet_aton(netmask))[0]
        broadcast_packed = ip_packed | ~mask_packed & 0xFFFFFFFF
        address = socket.inet_ntoa(struct.pack("!I", broadcast_packed))
        time_print(f"广播地址:{address}")
        return address
    except socket.error as e:
        time_print(f"\033[31m计算广播地址失败(无效IP或掩码):\033[0m {e}")
        return "255.255.255.255" # 回退地址
    except Exception as e:
        time_print(f"\033[31m计算广播地址失败:\033[0m {e}")
        return "255.255.255.255"  # 回退地址


# 发现局域网内的服务器
def discover_server(broadcast_ip, broadcast_port, time_out):
    servers = []
    # 创建UDP socket
    with socket.socket(socket.AF_INET, socket.SOCK_DGRAM) as sock:
        sock.setsockopt(socket.SOL_SOCKET, socket.SO_BROADCAST, 1)
        sock.settimeout(time_out)
        send_text = time_key("PODFLOW_DISCOVER_SERVER_REQUEST")
        send_text = send_text.encode("utf-8")
        try:
            # 发送广播请求
            sock.sendto(send_text, (broadcast_ip, broadcast_port))
        except Exception:
            time_print("\033[31m请求发送失败\033[0m", False, True, False)
            return servers
        # 等待响应
        start_time = time.time()
        while time.time() - start_time < time_out:
            try:
                data, addr = sock.recvfrom(1024)
                if data.startswith(b"PODFLOW_SERVER_INFO|"):
                    try:
                        port = int(data.decode().split("|")[1])
                        servers.append((addr[0], port))
                    except (IndexError, ValueError):
                        time_print("\033[31m响应格式错误\033[0m", False, True, False)
            except socket.timeout:
                break
            except Exception:
                time_print("\033[31m接收数据出错\033[0m", False, True, False)
                break
    return servers


# 自动发现并连接服务器模块
def connect_upload_server():
    # 如果配置中启用了上传功能
    if gVar.config["upload"]:
        upload_ip = gVar.config["upload_ip"]
        if upload_ip:
            broadcast_ip = upload_ip
        else:
            local_ip = get_local_ip()
            if not local_ip:
                # 如果无法获取本地IP，广播发现也无法进行
                time_print("\033[31m无法获取本地IP，跳过广播发现。\033[0m")
                return
            broadcast_ip = calculate_broadcast(local_ip)
            if broadcast_ip in ["255.255.255.255", local_ip, "0.0.0.0"]:
                # 避免向无效或自己的地址广播
                time_print(f"\033[31m计算出的广播地址无效或为本机地址: {broadcast_ip}，跳过广播发现。\033[0m")
                return
        # 打印正在搜索上传服务器
        time_print("正在搜索上传服务器...")
        # 服务器列表为空
        servers = []
        # 在允许的端口范围内尝试发现服务器
        progress_lock = threading.Lock()
        def try_port(port, servers):
            server = discover_server(broadcast_ip, port, TIMEOUT)
            with progress_lock:
                if server:
                    servers.extend(server)
                progress_update(0.0005, added=True)
        threads = []
        for port in range(BROADCAST_PORT, MAX_BROADCAST_PORT + 1):
            t = threading.Thread(target=try_port, args=(port, servers))
            t.start()
            threads.append(t)
        for t in threads:
            t.join()
        if not servers:
            time_print("找不到上传服务器", True)
        else:
            # 选择第一个找到的服务器
            server_ip, server_port = servers[0]
            time_print(f"正在连接到{server_ip}:{server_port}...", True)
            return f"http://{server_ip}:{server_port}"



# podflow/download/dl_aideo_video.py
# coding: utf-8

import os
import re
import ffmpeg
import yt_dlp
from podflow import gVar
from podflow.basic.write_log import write_log
from podflow.basic.time_print import time_print
from podflow.basic.get_duration import get_duration
from podflow.httpfs.download_bar import download_bar
from podflow.download.show_progress import show_progress
from podflow.message.fail_message_initialize import fail_message_initialize


# 下载视频模块
def download_video(
    video_url,
    output_dir,
    output_format,
    format_id,
    video_website,
    video_write_log,
    sesuffix="",
    cookies=None,
    playlist_num=None,
):
    class MyLogger:
        def debug(self, msg):
            pass

        def warning(self, msg):
            pass

        def info(self, msg):
            pass

        def error(self, msg):
            msg = fail_message_initialize(msg, video_url).ljust(48)
            time_print(msg, Top=True, Time=False)

    outtmpl = f"channel_audiovisual/{output_dir}/{video_url}{sesuffix}.{output_format}"
    ydl_opts = {
        "outtmpl": outtmpl,  # 输出文件路径和名称
        "format": f"{format_id}",  # 指定下载的最佳音频和视频格式
        "noprogress": True,
        "quiet": True,
        "progress_hooks": [show_progress],
        "logger": MyLogger(),
        "throttled_rate": "70K",  # 设置最小下载速率为:字节/秒
    }
    if cookies:
        if "www.bilibili.com" in video_website:
            ydl_opts["http_headers"] = {
                "User-Agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/118.0.0.0 Safari/537.36",
                "Referer": "https://www.bilibili.com/",
            }
        elif "www.youtube.com" in video_website:
            ydl_opts["http_headers"] = {
                "User-Agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/118.0.0.0 Safari/537.36",
                "Referer": "https://www.youtube.com/",
            }
        ydl_opts["cookiefile"] = cookies  # cookies 是你的 cookies 文件名
    if playlist_num:  # 播放列表的第n个视频
        ydl_opts["playliststart"] = playlist_num
        ydl_opts["playlistend"] = playlist_num
    try:
        with yt_dlp.YoutubeDL(ydl_opts) as ydl:
            ydl.download([f"{video_website}"])  # 下载指定视频链接的视频
        return None, None
    except Exception as download_video_error:
        fail_info = fail_message_initialize(download_video_error, video_url).replace(
            "\n", ""
        )
        remove_info = ""
        if (
            fail_info == ""
            or re.search(r"请求拒绝|数据不完整|传输中断|请求超时|响应超时", fail_info)
        ) and "www.youtube.com" in video_website:
            if fail_info != "":
                remove_info_part = "|"
            else:
                remove_info_part = ""
            if os.path.isfile(outtmpl):
                os.remove(outtmpl)
                remove_info = remove_info_part + "已删除失败文件"
            elif os.path.isfile(outtmpl + ".part"):
                os.remove(outtmpl + ".part")
                remove_info = remove_info_part + "已删除部分失败文件"
        write_log(
            f"{video_write_log} \033[31m下载失败\033[0m",
            None,
            True,
            True,
            f"错误信息: {fail_info}{remove_info}",
        )  # 写入下载失败的日志信息
        download_bar(mod=2, status="下载失败")
        return video_url, fail_info


# 视频完整下载模块
def dl_full_video(
    video_url,
    output_dir,
    output_format,
    format_id,
    id_duration,
    video_website,
    video_write_log,
    sesuffix="",
    cookies=None,
    playlist_num=None,
):
    video_id_failed, fail_info = download_video(
        video_url,
        output_dir,
        output_format,
        format_id,
        video_website,
        video_write_log,
        sesuffix,
        cookies,
        playlist_num,
    )
    if video_id_failed:
        return video_url, fail_info
    duration_video = get_duration(
        f"channel_audiovisual/{output_dir}/{video_url}{sesuffix}.{output_format}"
    )  # 获取已下载视频的实际时长
    if (
        duration_video is not None and abs(id_duration - duration_video) <= 1
    ):  # 检查实际时长与预计时长是否一致
        return None, None
    if duration_video:
        fail_info = f"不完整({id_duration}|{duration_video}"
        write_log(f"{video_write_log} \033[31m下载失败\033[0m\n错误信息: {fail_info})")
        download_bar(mod=2, status="下载失败")
        os.remove(
            f"channel_audiovisual/{output_dir}/{video_url}{sesuffix}.{output_format}"
        )  # 删除不完整的视频
    return video_url, fail_info


# 视频重试下载模块
def dl_retry_video(
    video_url,
    output_dir,
    output_format,
    format_id,
    id_duration,
    retry_count,
    video_website,
    video_write_log,
    sesuffix="",
    cookies=None,
    playlist_num=None,
):
    video_id_failed, _ = dl_full_video(
        video_url,
        output_dir,
        output_format,
        format_id,
        id_duration,
        video_website,
        video_write_log,
        sesuffix,
        cookies,
        playlist_num,
    )
    # 下载失败后重复尝试下载视频
    video_id_count = 0
    while video_id_count < retry_count and video_id_failed:
        if (
            cookies is None
            and "www.youtube.com" in video_website
            and gVar.youtube_cookie
        ):
            cookies = "channel_data/yt_dlp_youtube.txt"
        video_id_count += 1
        if cookies:
            write_log(
                f"{video_write_log} 第\033[34m{video_id_count}\033[0m次重新下载 🍪"
            )
        else:
            write_log(f"{video_write_log} 第\033[34m{video_id_count}\033[0m次重新下载")
        video_id_failed, _ = dl_full_video(
            video_url,
            output_dir,
            output_format,
            format_id,
            id_duration,
            video_website,
            video_write_log,
            sesuffix,
            cookies,
            playlist_num,
        )
    return video_id_failed


# 音视频总下载模块
def dl_aideo_video(
    video_url,
    output_dir,
    output_format,
    video_format,
    retry_count,
    video_website,
    output_dir_name="",
    cookies=None,
    playlist_num=None,
    display_color="\033[95m",
    title_name="",
):
    if output_dir_name:
        video_write_log = f"{display_color}{output_dir_name}\033[0m|{video_url}"
    else:
        video_write_log = video_url
    id_duration = video_format[0]
    print_message = (
        "\033[34m开始下载\033[0m 🍪" if cookies else "\033[34m开始下载\033[0m"
    )
    time_print(
        f"{video_write_log} {print_message}",
        NoEnter=True,
    )
    download_bar(
        mod=0,
        per=0,
        idname=output_dir_name,
        nametext=title_name,
        file=f"{video_url}.{output_format}",
    )
    if output_format == "m4a":
        if video_format[1] in ["140", "30280"]:
            time_print(
                "",
                Time=False,
            )
        else:
            time_print(
                f" \033[97m{video_format[1]}\033[0m",
                Time=False,
            )
        video_id_failed = dl_retry_video(
            video_url,
            output_dir,
            "m4a",
            video_format[1],
            id_duration,
            retry_count,
            video_website,
            video_write_log,
            "",
            cookies,
            playlist_num,
        )
    else:
        time_print(
            "",
            Time=False,
        )
        time_print(
            f"\033[34m视频部分开始下载\033[0m \033[97m{video_format[2]}\033[0m",
        )
        video_id_failed = dl_retry_video(
            video_url,
            output_dir,
            "mp4",
            video_format[2],
            id_duration,
            retry_count,
            video_website,
            video_write_log,
            ".part",
            cookies,
            playlist_num,
        )
        if video_id_failed is None:
            time_print(
                f"\033[34m音频部分开始下载\033[0m \033[97m{video_format[1]}\033[0m",
            )
            video_id_failed = dl_retry_video(
                video_url,
                output_dir,
                "m4a",
                video_format[1],
                id_duration,
                retry_count,
                video_website,
                video_write_log,
                ".part",
                cookies,
                playlist_num,
            )
        if video_id_failed is None:
            time_print(
                "\033[34m开始合成...\033[0m",
                NoEnter=True,
            )
            # 指定视频文件和音频文件的路径
            video_file = f"channel_audiovisual/{output_dir}/{video_url}.part.mp4"
            audio_file = f"channel_audiovisual/{output_dir}/{video_url}.part.m4a"
            output_file = f"channel_audiovisual/{output_dir}/{video_url}.mp4"
            try:
                # 使用 ffmpeg-python 合并视频和音频
                video = ffmpeg.input(video_file)
                audio = ffmpeg.input(audio_file)
                stream = ffmpeg.output(
                    audio, video, output_file, vcodec="copy", acodec="copy"
                )
                ffmpeg.run(stream, quiet=True)
                time_print(" \033[32m合成成功\033[0m", Time=False)
                # 删除临时文件
                os.remove(f"channel_audiovisual/{output_dir}/{video_url}.part.mp4")
                os.remove(f"channel_audiovisual/{output_dir}/{video_url}.part.m4a")
            except ffmpeg.Error as dl_aideo_video_error:
                video_id_failed = video_url
                time_print(
                    "",
                    Time=False,
                )
                write_log(
                    f"{video_write_log} \033[31m下载失败\033[0m\n错误信息: 合成失败:{dl_aideo_video_error}"
                )
    if video_id_failed is None:
        if output_format == "m4a":
            only_log = f" {video_format[1]}"
        else:
            only_log = f" {video_format[1]}+{video_format[2]}"
        if cookies:
            only_log += " Cookies"
        write_log(
            f"{video_write_log} \033[32m下载成功\033[0m", None, True, True, only_log
        )  # 写入下载成功的日志信息
        download_bar(mod=2, status="下载成功")
    return video_id_failed



# podflow/basic/split_dict.py
# coding: utf-8


# 字典拆分模块
def split_dict(data, chunk_size=100, firse_item_only=False):
    if chunk_size == 0:
        return [{}]
    end_value = chunk_size if firse_item_only else len(data)
    chunks = []
    for i in range(0, end_value, chunk_size):
        chunk = dict(list(data.items())[i:i+chunk_size])
        chunks.append(chunk)
    return chunks



# podflow/message/xml_rss.py
# coding: utf-8

import time


# 生成XML模块
def xml_rss(title, link, description, category, icon, items):
    # 获取当前时间
    current_time_now = time.time()  # 获取当前时间的秒数
    # 获取当前时区和夏令时信息
    time_info_now = time.localtime(current_time_now)
    # 构造时间字符串
    formatted_time_now = time.strftime("%a, %d %b %Y %H:%M:%S %z", time_info_now)
    #itunes_summary = description.replace("\n", "&#xA;")
    if title == "Podflow":
        author = "gruel-zxz"
        subtitle = "gruel-zxz-podflow"
    else:
        author = title
        subtitle = title
    return f"""<?xml version="1.0" encoding="UTF-8"?>
<rss version="2.0" xmlns:itunes="http://www.itunes.com/dtds/podcast-1.0.dtd">
    <channel>
        <title>{title}</title>
        <link>{link}</link>
        <description>{description}</description>
        <category>{category}</category>
        <generator>Podflow (support us at https://github.com/gruel-zxz/podflow)</generator>
        <language>en-us</language>
        <lastBuildDate>{formatted_time_now}</lastBuildDate>
        <pubDate>Sun, 24 Apr 2005 11:20:54 +0800</pubDate>
        <image>
            <url>{icon}</url>
            <title>{title}</title>
            <link>{link}</link>
        </image>
        <itunes:author>{author}</itunes:author>
        <itunes:subtitle>{subtitle}</itunes:subtitle>
        <itunes:summary><![CDATA[{description}]]></itunes:summary>
        <itunes:image href="{icon}"></itunes:image>
        <itunes:explicit>no</itunes:explicit>
        <itunes:category text="{category}"></itunes:category>
{items}
    </channel>
</rss>"""



# podflow/upload/linked_server.py
# coding: utf-8

import socket
from podflow.basic.write_log import write_log
from podflow.basic.time_print import time_print
from podflow.httpfs.port_judge import port_judge
from podflow.upload.time_key import check_time_key


# 定义一个函数，用于判断端口是否可用
def usable_port(port, max_num):
    # 定义主机IP地址
    hostip = "0.0.0.0"
    # 循环判断端口是否可用
    while port <= max_num:
        # 调用port_judge函数判断端口是否可用
        if port_judge(hostip, port):
            # 如果端口可用，则返回该端口
            return port
        else:
            # 如果端口不可用，则将端口加1
            port += 1
    # 如果循环结束后，仍然没有找到可用的端口，则返回None
    return None


# 处理服务发现请求的UDP服务模块
def handle_discovery(broadcast_port, service_port):
    # 创建UDP套接字
    with socket.socket(socket.AF_INET, socket.SOCK_DGRAM) as sock:
        # 设置套接字选项，允许地址重用
        sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        # 设置套接字选项，允许广播
        sock.setsockopt(socket.SOL_SOCKET, socket.SO_BROADCAST, 1)
        # 绑定套接字到广播端口
        sock.bind(("0.0.0.0", broadcast_port))
        # 打印发现服务已启动
        time_print(f"发现服务已启动|端口: \033[32m{broadcast_port}\033[0m")
        # 无限循环，等待接收广播消息
        while True:
            # 接收广播消息
            data, addr = sock.recvfrom(1024)
            # 将接收到的消息解码为字符串
            data = data.decode('utf-8')
            # 检查消息是否包含时间关键字
            if check_time_key(data ,"PODFLOW_DISCOVER_SERVER_REQUEST"):
                # 打印接收到的发现请求成功
                write_log(f"来自{addr[0]}的发现请求\033[32m成功\033[0m")
                # 构造响应消息
                response = f"PODFLOW_SERVER_INFO|{service_port}".encode()
                # 发送响应消息
                sock.sendto(response, addr)
            else:
                # 打印接收到的发现请求失败
                write_log(f"来自{addr[0]}的发现请求\033[31m失败\033[0m")



# podflow/upload/build_hash.py
# coding: utf-8

import hashlib


# 定义一个函数，用于计算文件的哈希值
def build_hash(file):
    # 获取hashlib模块中的sha256函数
    hash_func = getattr(hashlib, "sha256")()
    # 循环读取文件内容，每次读取4096字节
    for chunk in iter(lambda: file.read(4096), b''):
        # 更新哈希值
        hash_func.update(chunk)
    # 返回哈希值的十六进制表示
    return hash_func.hexdigest()



# coding: utf-8

from importlib.metadata import version
from podflow import parse
from podflow.main_upload import main_upload
from podflow.main_podcast import main_podcast
from podflow.basic.time_print import time_print
from podflow.repair.reverse_log import reverse_log
from podflow.parse_arguments import parse_arguments


def main():
    # 获取传入的参数
    parse_arguments()
    # 开始运行
    if parse.upload:
        time_print("Podflow|接收服务开始运行...")
        reverse_log("upload")
        main_upload()
    else:
        time_print(f"Podflow|{version('Podflow')} 开始运行...")
        reverse_log("Podflow")
        reverse_log("httpfs")
        main_podcast()


if __name__ == "__main__":
    main()



# podflow/remove/remove_dir.py
# coding: utf-8

import os
import re
import shutil
from podflow import gVar
from podflow.basic.write_log import write_log


# 删除已抛弃的媒体文件夹模块
def remove_dir():
    def remove_path(name):
        directory_path = f"channel_audiovisual/{name}"
        # 检查目录是否存在
        if os.path.exists(directory_path):
            # 删除该目录及其内容
            shutil.rmtree(directory_path)
        write_log(f"{name}文件夹已删除")

    folder_names = [
        folder
        for folder in os.listdir("channel_audiovisual")
        if os.path.isdir(f"channel_audiovisual/{folder}")
    ]
    folder_names_youtube = [name for name in folder_names if re.match(r"UC.{22}", name)]
    for name in folder_names_youtube:
        if name not in gVar.channelid_youtube_ids_original:
            remove_path(name)
    folder_names_bilibili = [name for name in folder_names if re.match(r"[0-9]+", name)]
    for name in folder_names_bilibili:
        if name not in gVar.channelid_bilibili_ids_original:
            remove_path(name)



# podflow/message/get_media_name.py
# coding: utf-8

import re


# 定义一个函数，用于获取媒体名称
def get_media_name(id_type, items):
    # 如果id_type为youtube
    if id_type == "youtube":
        # 使用正则表达式匹配items中的youtube链接，返回匹配结果
        return re.findall(
            r"(?:/UC.{22}/)(.{11}\.m4a|.{11}\.mp4)(?=\"|\?)",
            items,
        )
    # 如果id_type为bilibili
    elif id_type == "bilibili":
        # 使用正则表达式匹配items中的bilibili链接，返回匹配结果
        return re.findall(
            r"(?:/[0-9]+/)(BV.{10}\.m4a|BV.{10}\.mp4|BV.{10}_p[0-9]+\.m4a|BV.{10}_p[0-9]+\.mp4|BV.{10}_[0-9]{9}\.m4a|BV.{10}_[0-9]{9}\.mp4)(?=\"|\?)",
            items,
        )
    # 如果id_type不是youtube或bilibili
    else:
        # 返回空列表
        return []



# podflow/httpfs/get_channelid.py
# coding: utf-8

import re
from podflow.basic.http_client import http_client


def get_response(
    text,
    name,
    re_ucid,
    mod_ucid,
    re_title,
):
    if response := http_client(
        url=text,
        name=name,
        headers_possess=True
    ):
        if mod_ucid == "text":
            ucid = re.search(re_ucid, response.text)
        elif mod_ucid == "url":
            ucid = re.search(re_ucid, response.url)
        else:
            ucid = ""
        title = re.search(re_title, response.text)
        if ucid and title :
            ucid = ucid.group(0)
            title = title.group(0)
            return f'''"{title}": "{ucid}"

"{title}": {{
            "id": "{ucid}",
            "quality": "1080",
            "media": "mp4",
            "InmainRSS": false,
            "update_size": 2
        }}'''
        else:
            return "Network Error"
    else:
        return "Network Error"


def get_channelid(text):
    if text in [None, ""]:
        return ""
    url_pattern = re.compile(r'https?://[^\s<>"]+|www\.[^\s<>"]+')
    text_match = url_pattern.search(text)  # 使用 search() 而不是 match()
    if text_match:
        text = text_match.group(0)
        if "youtube" in text:
            return get_response(
                text,
                "youtube",
                r"(?<=https://www.youtube.com/channel/)UC.{22}",
                "text",
                r'(?<=<link itemprop="name" content=").+?(?=")',
            )
        elif "bilibili" in text or "b23.tv" in text:
            return get_response(
                text,
                "bilibili",
                r'(?<=https://space\.bilibili\.com/)([0-9]+)',
                "url",
                r'(?<=<title>).+(?=的个人空间-)',
            )
    else:
        text_match = re.search(r'(?<=UID:)([0-9]+)', text, re.IGNORECASE)
        if text_match:
            text = f"https://space.bilibili.com/{text_match.group(0)}"
            return get_response(
                text,
                "bilibili",
                r'(?<=https://space\.bilibili\.com/)([0-9]+)',
                "url",
                r'(?<=<title>).+(?=的个人空间-)',
            )
        else:
            return "Network Error"
    return "Network Error"



# podflow/message/display_qrcode_and_url.py
# coding: utf-8

from datetime import datetime
from podflow import gVar
from podflow.basic.qr_code import qr_code
from podflow.basic.time_print import time_print


# 显示网址及二维码模块
def display_qrcode_and_url(
    output_dir,
    display_rss_address,
    qrcode,
    name,
    ids_update,
):
    address = gVar.config["address"]
    if token := gVar.config["token"]:
        xml_url = f"{address}/channel_rss/{output_dir}.xml?token={token}"
    else:
        xml_url = f"{address}/channel_rss/{output_dir}.xml"

    if display_rss_address or output_dir in ids_update:
        update_text = "已更新" if output_dir in ids_update else "无更新"
        time_print(
            f"{name} 播客{update_text}|地址:",
            Url=xml_url,
        )
    if (
        (display_rss_address or output_dir in ids_update)
        and qrcode
        and output_dir not in gVar.displayed_QRcode
    ):
        qr_code(xml_url, True)
        gVar.displayed_QRcode.append(output_dir)



# podflow/download/delete_part.py
# coding: utf-8

import os
import fnmatch
from podflow.basic.write_log import write_log


# 删除下载失败媒体模块
def delete_part(channelid_ids):
    relative_path = "channel_audiovisual"
    parent_folder_path = os.path.abspath(relative_path)
    for root, _, filenames in os.walk(parent_folder_path):
        for filename in fnmatch.filter(filenames, '*.part'):
            file_path = os.path.join(root, filename)
            os.remove(file_path)
            ids = os.path.basename(root).split('/')[-1]
            if ids in channelid_ids:
                ids = channelid_ids[ids]
            write_log(f"{ids}|{filename}已删除")



# podflow/basic/time_format.py
# coding: utf-8


# 格式化时间模块
def time_format(duration):
    if duration is None:
        return "Unknown"
    duration = int(duration)
    hours, remaining_seconds = divmod(duration, 3600)
    minutes = remaining_seconds // 60
    remaining_seconds = remaining_seconds % 60
    if hours > 0:
        return "{:02}:{:02}:{:02}".format(hours, minutes, remaining_seconds)
    else:
        return "{:02}:{:02}".format(minutes, remaining_seconds)



# podflow/basic/time_print.py
# coding: utf-8

from datetime import datetime
from podflow import gVar
from podflow.httpfs.to_html import ansi_to_html, qrcode_to_html


def time_print(
    text,
    Top=False,
    NoEnter=False,
    Time=True,
    Url="",
    Qrcode=False,
    Number=None,
    Head="",
):
    if Time:
        text = f"{datetime.now().strftime('%H:%M:%S')}|{text}"
    text_print = f"\r{text}" if Top else f"{text}"
    if Url:
        text_print = f"{text_print}\n\033[34m{Url}\033[0m"
    if Head:
        text_print = f"{Head}{text_print}"
    if NoEnter:
        print(text_print, end="")
    else:
        print(text_print)

    if Number is not None and (
        not isinstance(Number, int)
        or not -min(len(gVar.index_message["podflow"]), 4) <= Number < 0
    ):
        Number = None
    if text:
        text = qrcode_to_html(Qrcode) if Qrcode else ansi_to_html(text)
        if Number:
            gVar.index_message["podflow"][Number] = text
        elif not gVar.index_message["enter"] and gVar.index_message["podflow"]:
            if Top:
                gVar.index_message["podflow"][-1] = text
            else:
                gVar.index_message["podflow"][-1] += text
        else:
            gVar.index_message["podflow"].append(text)
    gVar.index_message["enter"] = not NoEnter
    if Url:
        gVar.index_message["podflow"].append(
            f'<a href="{Url}" target="_blank"><span class="ansi-url">{Url}</span></a>'
        )



# podflow/httpfs/port_judge.py
# coding: utf-8

import socket


# 定义一个函数，用于判断指定IP地址和端口号是否可用
def port_judge(hostip, port):
    # 尝试创建一个socket对象
    try:
        # 创建一个socket对象，指定地址族为IPv4，类型为TCP
        with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as sock:
            # 设置socket选项，允许地址重用
            sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
            # 绑定socket对象到指定的IP地址和端口号
            sock.bind((hostip, port))
            # 如果绑定成功，返回True
            return True
    # 如果绑定失败，捕获OSError异常，并返回False
    except OSError:
        return False



# podflow/upload/add_upload.py
# coding: utf-8

import time
from podflow import gVar


# 添加新媒体至上传列表模块
def add_upload():
    # 获取video_id_update_format和video_id_failed的值
    video_id_update_format = gVar.video_id_update_format
    video_id_failed = gVar.video_id_failed
    # 遍历video_id_update_format的键值对
    for video_id, video_id_value in video_id_update_format.items():
        # 判断video_id_value是否为字典，并且main不在video_id_failed中
        if (
            isinstance(video_id_value, dict)
            and video_id_value["main"] not in video_id_failed
        ):
            # 构造media_id
            media_id = f"{video_id}.{video_id_value['media']}"
            # 判断gVar.upload_original中是否存在media_id
            if not any(
                item.get("media_id") == media_id for item in gVar.upload_original
            ):
                # 如果不存在，则将media_id、channel_id、media_time、upload、remove、hash添加到gVar.upload_original中
                gVar.upload_original.append(
                    {
                        "media_id": media_id,
                        "channel_id": video_id_value["id"],
                        "media_time": int(time.time()),
                        "upload": False,
                        "remove": False,
                        "hash": None,
                    }
                )




# podflow/basic/write_log.py
# coding: utf-8

import re
from datetime import datetime
from podflow.basic.time_print import time_print


# 日志模块
def write_log(
    log,
    suffix=None,
    display=True,
    time_display=True,
    only_log="",
    file_name="Podflow.log",
    url=""
):
    # 获取当前的具体时间
    current_time = datetime.now()
    # 格式化输出, 只保留年月日时分秒
    formatted_time = current_time.strftime("%Y-%m-%d %H:%M:%S")
    # 将新的日志内容添加在原有内容之前
    log_in = re.sub(r"\033\[[0-9;]+m", "", log)
    log_in = re.sub(r"\n", "", log_in)
    only_log = re.sub(r"\033\[[0-9;]+m", "", str(only_log)) if only_log else ""
    new_contents = (
        f"{formatted_time} {log_in}{only_log}\n"
        if only_log
        else f"{formatted_time} {log_in}\n"
    )
    # 将新的日志内容写入文件
    with open(file_name, "a", encoding="utf-8") as file:
        file.write(new_contents)
    if display:
        formatted_time_mini = current_time.strftime("%H:%M:%S")
        log_print = f"{formatted_time_mini}|{log}" if time_display else f"{log}"
        log_print = f"{log_print}|{suffix}" if suffix else f"{log_print}"
        time_print(log_print, Time=False, Url=url)



# podflow/__init__.py
# coding: utf-8

# 默认参数
default_config = {
    "preparation_per_count": 100,  # 获取媒体信息每组数量
    "completion_count": 100,  # 媒体缺失时最大补全数量
    "retry_count": 5,  # 媒体下载重试次数
    "url": "http://127.0.0.1",  # HTTP共享地址
    "port": 8000,  # HTTP共享端口
    "port_in_url": True,  # HTTP共享地址是否包含端口
    "httpfs": False,  # HTTP共享日志
    "title": "Podflow",  # 博客的名称
    "filename": "Podflow",  # 主XML的文件名称
    "link": "https://github.com/gruel-zxz/podflow",  # 博客主页
    "description": "在iOS平台上借助workflow和a-shell搭建专属的播客服务器。",  # 博客信息
    "icon": "https://raw.githubusercontent.com/gruel-zxz/podflow/main/Podflow.png",  # 博客图标
    "category": "TV &amp; Film",  # 博客类型
    "token": "",  # token认证, 如为null或""将不启用token
    "delete_incompletement": False,  # 是否删除下载中断媒体(下载前处理流程)
    "remove_media": True,  # 是否删除无用的媒体文件
    "upload": False,  # 是否将长期媒体进行上传
    "upload_ip": "10.0.3.231",  # 长期媒体进行上传服务器地址(可不写, 会自动搜索, 无法搜索请填写)
    "channelid_youtube": {  # Youtube频道列表
        "youtube": {
            "update_size": 15,  # 每次获取频道媒体数量
            "id": "UCBR8-60-B28hp2BmDPdntcQ",  # 频道ID
            "title": "YouTube",  # 频道名称
            "quality": "480",  # 媒体分辨率(仅在media为视频时有效)
            "last_size": 50,  # 媒体保留数量
            "media": "m4a",  # 下载媒体类型
            "DisplayRSSaddress": False,  # 是否在Print中显示子博客地址
            "InmainRSS": True,  # 是否在主博客中
            "QRcode": False,  # 是否显示子博客地址二维码(仅在DisplayRSSaddress为True时有效)
            "BackwardUpdate": False,  # 是否向后更新
            "BackwardUpdate_size": 3,  # 向后更新数量(仅在BackwardUpdate为True时有效)
            "want_retry_count": 8,  # 媒体获取失败后多少次后重试(小于等于该数量时将一直重试)
            "title_change": [  # 标题文本修改(默认为无, 可多个条件，以列表形式存在)
                {  # match和url参数至少有一个, 如都有将同时生效
                    "mode": "add-left",  # 修改模式(add-left: 开头添加, add-right: 结尾添加, replace: 内容替换)
                    "match": "",  # 需要匹配的规则(为正则表达式)
                    "url": "https://www.youtube.com/playlist?list=...",  # 播放列表网址(只适用于YouTube频道, 并且不适用replace模式, 选择后会失效)
                    "text": "",  # 需要替换或添加的文本
                },
                {
                    "mode": "add-right",
                    "match": "",
                    "url": "",
                    "text": "",
                },
            ],
            "NoShorts": False,  # 是否不下载Shorts媒体
        },
    },
    "channelid_bilibili": {  # 哔哩哔哩频道列表
        "哔哩哔哩弹幕网": {
            "update_size": 25,
            "id": "8047632",
            "title": "哔哩哔哩弹幕网",
            "quality": "480",
            "last_size": 100,
            "media": "m4a",
            "DisplayRSSaddress": False,
            "InmainRSS": True,
            "QRcode": False,
            "BackwardUpdate": False,
            "BackwardUpdate_size": 3,
            "want_retry_count": 8,
            "title_change": {
                "mode": "replace",
                "match": "",
                "text": "",
            },
            "AllPartGet": False,  # 是否提前获取分P或互动视频(建议update_size大于5时使用, 如果该变量不存在时, 默认update_size大于5时开启)
        },
    },
}
# 如果InmainRSS为False或频道有更新则无视DisplayRSSaddress的状态, 都会变为True。


# 全局变量
class Application_gVar:
    def __init__(self):
        self.config = {}  # 配置文件字典
        self.channelid_youtube = {}  # YouTube频道字典
        self.channelid_bilibili = {}  # 哔哩哔哩频道字典
        self.channelid_youtube_ids = {}  # YouTube频道ID字典
        self.channelid_youtube_ids_original = {}  # 原始YouTube频道ID字典
        self.channelid_bilibili_ids = {}  # 哔哩哔哩频道ID字典
        self.channelid_bilibili_ids_original = {}  # 原始哔哩哔哩频道ID字典

        self.server_process_print_flag = ["keep"]  # httpserver进程打印标志列表
        self.update_generate_rss = True  # 更新并生成rss布朗值
        self.displayed_QRcode = []  # 已显示二维码列表

        self.bilibili_data = {}  # 哔哩哔哩data字典
        self.youtube_cookie = {}  # YouTube cookie字典
        self.channelid_youtube_ids_update = {}  # 需更新的YouTube频道字典
        self.youtube_content_ytid_update = {}  # 需下载YouTube视频字典
        self.youtube_content_ytid_backward_update = {}  # 向后更新需下载YouTube视频字典
        self.channelid_youtube_rss = {}  # YouTube频道最新Rss Response字典
        self.channelid_bilibili_ids_update = {}  # 需更新的哔哩哔哩频道字典
        self.bilibili_content_bvid_update = {}  # 需下载哔哩哔哩视频字典
        self.channelid_bilibili_rss = {}  # 哔哩哔哩频道最新Rss Response字典
        self.bilibili_content_bvid_backward_update = {}  # 向后更新需下载哔哩哔哩视频字典
        self.video_id_failed = []  # YouTube&哔哩哔哩视频下载失败列表
        self.video_id_update_format = {}  # YouTube和哔哩哔哩视频下载的详细信息字典
        self.hash_rss_original = ""  # 原始rss哈希值文本
        self.xmls_original = {}  # 原始xml信息字典
        self.xmls_original_fail = []  # 未获取原始xml频道列表
        self.xmls_quantity = {}  # xml数量字典
        self.youtube_xml_get_tree = {}  # YouTube频道简介和图标字典
        self.all_youtube_content_ytid = {}  # 所有YouTube视频id字典
        self.all_bilibili_content_bvid = {}  # 所有哔哩哔哩视频id字典
        self.all_items = {}  # 更新后所有item明细字典
        self.overall_rss = ""  # 更新后的rss文本
        self.make_up_file_format = {}  # 补全缺失媒体字典
        self.make_up_file_format_fail = {}  # 补全缺失媒体失败字典

        self.upload_original = []  # 原始上传信息字典
        self.upload_data = {}  # 上传用户账号密码字典

        self.shortcuts_url = {}  # 输出至shortcut的url字典

        self.index_message = {  # 图形界面显示信息字典
            "podflow": [],  # 主窗口信息列表
            "http": [],  # httpfs窗口信息列表
            "enter": True,  # 是否换行
            "schedule": [0, "准备中"],  # 进度条信息列表
            "download":[],  # 下载信息列表
        }


# 参数变量
class Application_parse:
    def __init__(self):
        self.shortcuts_url_original = []
        self.argument = ""
        self.update_num = -1
        self.time_delay = 0
        self.config = ""
        self.period = 1
        self.file = ""
        self.httpfs = False
        self.index = False


# 创建 Application 类的实例
gVar = Application_gVar()
parse = Application_parse()



# podflow/config/correct_config.py
# coding: utf-8

import re
from podflow import gVar, default_config, parse


# 纠正配置信息config模块
def correct_config():
    config = gVar.config
    # 对completion_count进行纠正
    if (
        "completion_count" not in config
        or not isinstance(config["completion_count"], int)
        or config["completion_count"] < 0
    ):
        config["completion_count"] = default_config["completion_count"]
    # 对preparation_per_count进行纠正
    if (
        "preparation_per_count" not in config
        or not isinstance(config["preparation_per_count"], int)
        or config["preparation_per_count"] <= 0
    ):
        config["preparation_per_count"] = default_config["preparation_per_count"]
    # 对retry_count进行纠正
    if (
        "retry_count" not in config
        or not isinstance(config["retry_count"], int)
        or config["retry_count"] <= 0
    ):
        config["retry_count"] = default_config["retry_count"]
    # 对url进行纠正
    match_url = re.search(r"^(https?|ftp)://([^/\s:]+)", config["url"])
    if "url" in config and match_url:
        config["url"] = match_url.group()
    else:
        config["url"] = default_config["url"]
    # 对port进行纠正
    if (
        "port" not in config
        or not isinstance(config["port"], int)
        or config["port"] < 0
        or config["port"] > 65535
    ):
        config["port"] = default_config["port"]
    # 对port_in_url进行纠正
    if "port_in_url" not in config or not isinstance(config["port_in_url"], bool):
        config["port_in_url"] = default_config["port_in_url"]
    # 合并地址和端口
    if config["port_in_url"]:
        config["address"] = f"{config['url']}:{config['port']}"
    else:
        config["address"] = config["url"]
    # 对httpfs进行纠正
    if "httpfs" not in config or not isinstance(config["httpfs"], bool):
        config["httpfs"] = default_config["httpfs"]
    # 对title进行纠正
    if "title" not in config:
        config["title"] = default_config["title"]
    # 对filename进行纠正
    if "filename" not in config:
        config["filename"] = default_config["filename"]
    # 对link进行纠正
    if "link" not in config or not re.search(
        r"^(https?|ftp)://[^\s/$.?#].[^\s]*$", config["link"]
    ):
        config["link"] = default_config["link"]
    # 对description进行纠正
    if "description" not in config:
        config["description"] = default_config["description"]
    # 对icon进行纠正
    if "icon" not in config or not re.search(
        r"^(https?|ftp)://[^\s/$.?#].[^\s]*$", config["icon"]
    ):
        config["icon"] = default_config["icon"]
    # 对category进行纠正
    if "category" not in config:
        config["category"] = default_config["category"]
    # 对IOS中shortcuts自动关注进行纠正
    if (
        f"{config['address']}/{config['filename']}.xml"
        not in parse.shortcuts_url_original
    ):
        gVar.shortcuts_url[f"{config['filename']}(Main RSS)"] = (
            f"{config['address']}/{config['filename']}.xml"
        )
    # 对token进行纠正
    if "token" not in config:
        config["token"] = default_config["token"]
    if config["token"] in [None, ""]:
        config["token"] = ""
    else:
        config["token"] = str(config["token"])
    # 对delete_incompletement进行纠正
    if "delete_incompletement" not in config or not isinstance(
        config["delete_incompletement"], bool
    ):
        config["delete_incompletement"] = default_config["delete_incompletement"]
    # 对remove_media进行纠正
    if "remove_media" not in config or not isinstance(
        config["remove_media"], bool
    ):
        config["remove_media"] = default_config["remove_media"]
    # 对upload进行纠正
    if "upload" not in config or not isinstance(
        config["upload"], bool
    ):
        config["upload"] = default_config["upload"]
    # 对upload_ip进行纠正
    if "upload_ip" not in config:
        config["upload_ip"] = ""
    else:
        config["upload_ip"] = str(config["upload_ip"])
